from os import path, remove, sep
from pathlib import Path
from shutil import rmtree

from pyrogram import filters
from pyrogram.client import Client
from pyrogram.types import CallbackQuery, InlineKeyboardMarkup, InlineKeyboardButton

from modules.api_client import upload_pic
from modules.app import app
from modules.logger import logWrite
from modules.submissions import subBlock, subUnblock
from modules.utils import configGet, jsonLoad, jsonSave, locale
from modules.database import col_submitted
from bson import ObjectId


@app.on_callback_query(filters.regex("sub_yes_[\s\S]*"))
async def callback_query_yes(app: Client, clb: CallbackQuery):

    fullclb = clb.data.split("_")
    user_locale = clb.from_user.language_code
    
    db_entry = col_submitted.find_one({"_id": ObjectId(fullclb[2])})
    submission = None

    if db_entry is None:
        await clb.answer(text=locale("sub_msg_unavail", "callback", locale=user_locale), show_alert=True)
        return
    else:
        if db_entry["temp"]["uuid"] is not None:
            if not path.exists(path.join(configGet("data", "locations"), "submissions", db_entry["temp"]["uuid"], db_entry["temp"]["file"])):
                await clb.answer(text=locale("sub_msg_unavail", "callback", locale=user_locale), show_alert=True)
                return
            else:
                filepath = path.join(configGet("data", "locations"), "submissions", db_entry["temp"]["uuid"], db_entry["temp"]["file"])
        else:
            try:
                submission = await app.get_messages(db_entry["user"], db_entry["telegram"]["msg_id"])
                filepath = await app.download_media(submission, file_name=configGet("tmp", "locations")+sep)
            except:
                await clb.answer(text=locale("sub_msg_unavail", "callback", locale=user_locale), show_alert=True)
                return

    response = await upload_pic(str(filepath))

    if response[0] is False and len(response[1]) == 0:
        await clb.answer(text=locale("sub_upload_failed", "callback", locale=user_locale), show_alert=True)
        return
    elif response[0] is False:
        await clb.answer(text=locale("sub_duplicates_found", "callback", locale=user_locale), show_alert=True)
        await clb.message.reply_text(locale("sub_media_duplicates_list", "message", locale=user_locale).format("\n • ".join(response[1])))
        return

    col_submitted.find_one_and_update({"_id": ObjectId(fullclb[2])}, {"$set": {"done": True}})

    try:
        if db_entry["temp"]["uuid"] is not None:
            rmtree(path.join(configGet("data", "locations"), "submissions", db_entry["temp"]["uuid"]), ignore_errors=True)
        else:
            remove(str(filepath))
    except (FileNotFoundError, NotADirectoryError):
        logWrite(f"Could not delete '{filepath}' on submission accepted", debug=True)

    if submission is not None:
        await submission.reply_text(locale("sub_yes", "message", locale=submission.from_user.language_code), quote=True)
    else:
        await app.send_message(db_entry["user"], locale("sub_yes", "message"))

    await clb.answer(text=locale("sub_yes", "callback", locale=user_locale).format(fullclb[2]), show_alert=True)

    edited_markup = [[InlineKeyboardButton(text=str(locale("accepted", "button")), callback_data="nothing")], clb.message.reply_markup.inline_keyboard[1]] if len(clb.message.reply_markup.inline_keyboard) > 1 else [[InlineKeyboardButton(text=str(locale("accepted", "button")), callback_data="nothing")]]
    await clb.message.edit(text=clb.message.text, reply_markup=InlineKeyboardMarkup(edited_markup))

    # Change keyboard to a completed variant
    
    # Send replies to both user and admin about accepting the application

    # try:
    #     if configGet("api_based", "mode") is True:
    #         media = await app.download_media(submission, file_name=configGet("tmp", "locations")+sep)
    #         upload = upload_pic(media)
    #         if upload[0] is False:
    #             await clb.answer(text=locale("sub_media_failed", "message", locale=user_locale), show_alert=True)
    #         elif len(upload[1]) > 0:
    #             await clb.answer(text=locale("sub_media_duplicates", "message", locale=user_locale))
    #             await clb.message.reply_text(locale("sub_media_duplicates_list", "message", locale=user_locale).format("\n • ".join(upload[1])))
    #         else:
    #             if clb.data.endswith("_caption"):
    #                 index = jsonLoad(configGet("index", "locations"))
    #                 index["captions"][Path(media).name] = submission.caption
    #                 jsonSave(index, configGet("index", "locations"))
    #     else:
    #         media = await app.download_media(submission, file_name=configGet("queue", "locations")+sep)
    #         if clb.data.endswith("_caption"):
    #             index = jsonLoad(configGet("index", "locations"))
    #             index["captions"][Path(media).name] = submission.caption
    #             jsonSave(index, configGet("index", "locations"))
    # except:
    #     await clb.answer(text=locale("sub_media_unavail", "message", locale=user_locale), show_alert=True)
    #     return
    # await submission.reply_text(locale("sub_yes", "message", locale=submission.from_user.language_code), quote=True)
    # await clb.answer(text=locale("sub_yes", "callback", locale=user_locale).format(fullclb[2]), show_alert=True)

    # edited_markup = [[InlineKeyboardButton(text=str(locale("accepted", "button")), callback_data="nothing")]]
    # await clb.message.edit(text=clb.message.text, reply_markup=InlineKeyboardMarkup(edited_markup))


@app.on_callback_query(filters.regex("sub_no_[\s\S]*"))
async def callback_query_no(app: Client, clb: CallbackQuery):
    fullclb = clb.data.split("_")
    user_locale = clb.from_user.language_code
    try:
        submission = await app.get_messages(int(fullclb[2]), int(fullclb[3]))
    except:
        await clb.answer(text=locale("sub_msg_unavail", "message", locale=user_locale), show_alert=True)
        return
    await submission.reply_text(locale("sub_no", "message", locale=submission.from_user.language_code), quote=True)
    await clb.answer(text=locale("sub_no", "callback", locale=user_locale).format(fullclb[2]), show_alert=True)

    # edited_markup = [[InlineKeyboardButton(text=str(locale("accepted", "button")), callback_data="nothing")]]
    # await clb.message.edit(text=clb.message.text, reply_markup=InlineKeyboardMarkup(edited_markup))


@app.on_callback_query(filters.regex("sub_block_[\s\S]*"))
async def callback_query_block(app: Client, clb: CallbackQuery):
    fullclb = clb.data.split("_")
    user_locale = clb.from_user.language_code
    await app.send_message(int(fullclb[2]), locale("sub_msg_unavail", "message", locale=configGet("locale")))
    subBlock(int(fullclb[2]))
    await clb.answer(text=locale("sub_block", "callback", locale=user_locale).format(fullclb[2]), show_alert=True)

    # edited_markup = [[InlineKeyboardButton(text=str(locale("accepted", "button")), callback_data="nothing")]]
    # await clb.message.edit(text=clb.message.text, reply_markup=InlineKeyboardMarkup(edited_markup))


@app.on_callback_query(filters.regex("sub_unblock_[\s\S]*")) 
async def callback_query_unblock(app: Client, clb: CallbackQuery):
    fullclb = clb.data.split("_")
    user_locale = clb.from_user.language_code
    await app.send_message(int(fullclb[2]), locale("sub_msg_unavail", "message", locale=configGet("locale")))
    subUnblock(int(fullclb[2]))
    await clb.answer(text=locale("sub_unblock", "callback", locale=user_locale).format(fullclb[2]), show_alert=True)

    # edited_markup = [[InlineKeyboardButton(text=str(locale("accepted", "button")), callback_data="nothing")]]
    # await clb.message.edit(text=clb.message.text, reply_markup=InlineKeyboardMarkup(edited_markup))